#!/usr/bin/env python
# -*- coding: utf-8 -*-
import datetime
from os.path import join

import luigi

from crypta.lib.python.yt import yt_helpers
from crypta.profile.lib.socdem_helpers.inference_utils import inference
from crypta.profile.utils.config import config
from crypta.profile.utils.loggers import TimeTracker
from crypta.profile.utils.luigi_utils import (
    BaseYtTask,
    YtTarget,
)
from crypta.profile.tasks.features.get_app_metrica_data import (
    GetDailyDevidVectors,
    GetDailyAppByDevid,
)


class GetBasicDevidProfiles(BaseYtTask):
    date = luigi.Parameter()
    priority = 100
    task_group = 'export_profiles'
    resource_type = 'CRYPTA_MOBILE_SOCDEM_MODELS'

    def requires(self):
        return {
            'daily_devid_vectors': GetDailyDevidVectors(self.date),
            'app_metrica': GetDailyAppByDevid(self.date),
        }

    def output(self):
        return YtTarget(join(config.RAW_DEVID_PROFILES_YT_DIRECTORY, self.date))

    def run(self):
        with TimeTracker(self.__class__.__name__):
            with self.yt.TempTable() as neuro_raw_devid_profiles:
                released = 'stable' if config.environment == 'production' else 'testing'

                inference.get_nn_predictions_for_all_profiles(
                    yt_client=self.yt,
                    is_mobile=True,
                    neuro_raw_profiles=neuro_raw_devid_profiles,
                    logger=self.logger,
                    resource_type=self.resource_type,
                    released=released,
                    date=self.date,
                )
                inference.get_catboost_predictions_for_all_profiles(
                    yt_client=self.yt,
                    yql_client=self.yql.yql_client,
                    is_mobile=True,
                    neuro_raw_profiles=neuro_raw_devid_profiles,
                    raw_profiles=self.output().table,
                    logger=self.logger,
                    resource_type=self.resource_type,
                    released=released,
                    date=self.date,
                )

                yt_helpers.set_ttl(
                    self.output().table,
                    datetime.timedelta(days=config.NUMBER_OF_INTERMEDIATE_PROFILES_TABLES_TO_KEEP),
                    yt_client=self.yt,
                )
